﻿local L = AceLibrary("AceLocale-2.2"):new("GridStatusGrace")
local GridRoster = GridRoster

local spellNameGrace = GetSpellInfo(47516)

GridStatusGrace = GridStatus:NewModule("GridStatusGrace")
GridStatusGrace.menuName = spellNameGrace

L:RegisterTranslations("enUS", function()
	return {
		["Grace Stack"] = true,
		["Grace Duration"] = true,

		["Color 1"] = true,
		["Color 2"] = true,
		["Color 3"] = true,

		["Color when player has one counter of grace."] = true,
		["Color when player has two counters of grace."] = true,
		["Color when player has three counters of grace."] = true,

		["Global Cooldown Mode"] = true,
		["Global Cooldown Duration"] = true,
		["Global Cooldown Delay"] = true,
	}
end)

GridStatusGrace.defaultDB = {
	debug = false,
	alert_GraceStack = {
		text = L["Grace Stack"],
		enable = true,
		color = { r = 0, g = 1, b = 1, a = 1 },
		color2 = { r = .5, g = .75, b = 0, a = 1 },
		color3 = { r = 1, g = .5, b = 0, a = 1 },
		gcdMode = false,
		gcdDuration = 1.5,
		gcdDelay = 0.5,
		priority = 70,
		range = false,
	},
	alert_GraceDuration = {
		text = L["Grace Duration"],
		enable = true,
		color = { r = 1, g = 1, b = 0, a = 1 },
		color2 = { r = 0, g = 1, b = 1, a = 1 },
		color3 = { r = 0, g = 1, b = 0, a = 1 },
		priority = 70,
		range = false,
	},
}

local alert_GraceStackOptions = {
	["color2"] = {
		type = "color",
		order = 100,
		name = L["Color 2"],
		desc = L["Color when player has two counters of grace."],
		hasAlpha = true,
		get = function ()
			local color = GridStatusGrace.db.profile.alert_GraceStack.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusGrace.db.profile.alert_GraceStack.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color3"] = {
		type = "color",
		order = 101,
		name = L["Color 3"],
		desc = L["Color when player has three counters of grace."],
		hasAlpha = true,
		get = function ()
			local color = GridStatusGrace.db.profile.alert_GraceStack.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusGrace.db.profile.alert_GraceStack.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local alert_GraceDurationOptions = {
	["color2"] = {
		type = "color",
		order = 100,
		name = L["Color 2"],
		desc = L["Color when player has two counters of grace."],
		hasAlpha = true,
		get = function ()
			local color = GridStatusGrace.db.profile.alert_GraceDuration.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusGrace.db.profile.alert_GraceDuration.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color3"] = {
		type = "color",
		order = 101,
		name = L["Color 3"],
		desc = L["Color when player has three counters of grace."],
		hasAlpha = true,
		get = function ()
			local color = GridStatusGrace.db.profile.alert_GraceDuration.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusGrace.db.profile.alert_GraceDuration.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["gcdMode"] = {
		type = "toggle",
		order = 102,
		name = L["Global Cooldown Mode"],
		desc = L["Global Cooldown Mode"]..".",
		get = function ()
			return GridStatusGrace.db.profile.alert_GraceStack.gcdMode
		end,
		set = function (arg)
			GridStatusGrace.db.profile.alert_GraceStack.gcdMode = arg
		end,
	},
	["gcdDuration"] = {
		type = "range",
		order = 103,
		name = L["Global Cooldown Duration"],
		desc = L["Global Cooldown Duration"]..".",
		min = 1.5,
		max = 2,
		step = 0.01,
		disabled = function()
			if GridStatusGrace.db.profile.alert_GraceStack.gcdMode then
				return false
			else
				return true
			end
		end,
		get = function ()
			return GridStatusGrace.db.profile.alert_GraceStack.gcdDuration
		end,
		set = function (arg)
			GridStatusGrace.db.profile.alert_GraceStack.gcdDuration = arg
		end,
	},
	["gcdDelay"] = {
		type = "range",
		order = 104,
		name = L["Global Cooldown Delay"],
		desc = L["Global Cooldown Delay"]..".",
		min = 0,
		max = 1,
		step = 0.01,
		disabled = function()
			if GridStatusGrace.db.profile.alert_GraceStack.gcdMode then
				return false
			else
				return true
			end
		end,
		get = function ()
			return GridStatusGrace.db.profile.alert_GraceStack.gcdDelay
		end,
		set = function (arg)
			GridStatusGrace.db.profile.alert_GraceStack.gcdDelay = arg
		end,
	}
}

local format = string.format

function GridStatusGrace:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatuses()
end

function GridStatusGrace:OnEnable()
	self.debugging = self.db.profile.debug
	self:Debug("OnEnable")

	self.super.OnEnable(self)
end

function GridStatusGrace:Reset()
	self.super.Reset(self)

	self:UnregisterStatuses()
	self:RegisterStatuses()
	self:UpdateAllUnits()
end


function GridStatusGrace:RegisterStatuses()
	self:RegisterStatus("alert_GraceStack", L["Grace Stack"], alert_GraceStackOptions)
	self:RegisterStatus("alert_GraceDuration", L["Grace Duration"], alert_GraceDurationOptions)
end

function GridStatusGrace:UnregisterStatuses()
	self:UnregisterStatus("alert_GraceStack")
	self:UnregisterStatus("alert_GraceDuration")
end


function GridStatusGrace:Grid_UnitJoined(name, unitid)
	self:UpdateUnit(unitid)
end

function GridStatusGrace:UpdateAllUnits()
	for guid, unitid in GridRoster:IterateRoster() do
		self:UpdateUnit(unitid)
	end
end


-- expirationTimeList[guid] = expirationTime
-- colorList[guid] = color
local activeCount = 0
local expirationTimeList = {}
local colorList = {}

function GridStatusGrace:RefreshActiveUnits()
	local settings = self.db.profile.alert_GraceDuration
	local now = GetTime()
	activeCount = 0

	for guid, expirationTime in pairs(expirationTimeList) do
		if (expirationTime > now) then
			local timeLeft = expirationTime - now
			local color = colorList[guid]
			activeCount = activeCount + 1

			if (timeLeft < 0) then
				timeLeft = 0
			end

			if settings.gcdMode then
				local gcd_count = (timeLeft - settings.gcdDelay) / settings.gcdDuration
				if (gcd_count < 0) then
					gcd_count = 0
				end
				local max_gcd_count = (7 - settings.gcdDelay) / settings.gcdDuration

				self.core:SendStatusGained(guid, "alert_GraceDuration",
										   settings.priority,
										   (settings.range and 40),
										   color,
										   format("%.1f", gcd_count),
										   gcd_count,
										   max_gcd_count)
			else
				self.core:SendStatusGained(guid, "alert_GraceDuration",
										   settings.priority,
										   (settings.range and 40),
										   color,
										   format("%.1f", timeLeft),
										   timeLeft,
										   6)
			end
		else
			expirationTimeList[guid] = nil
			colorList[guid] = nil

			self.core:SendStatusLost(guid, "alert_GraceDuration")
		end
	end

	-- no units with Grace on them?  cancel the repeating event
	if (activeCount == 0) then
		self:CancelScheduledEvent("GridGrace_Refresh")
	end
end


local f = function()
	GridStatusGrace:RefreshActiveUnits()
end

function GridStatusGrace:UpdateUnit(unitid, ignoreRange)
	local guid = UnitGUID(unitid)
	if (not GridRoster:IsGUIDInRaid(guid)) then
		return
	end

	local settings = self.db.profile.alert_GraceStack
	local durationEnabled = self.db.profile.alert_GraceDuration.enable
	local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitAura(unitid, spellNameGrace, nil, "HELPFUL|PLAYER")

	if (isMine and duration) then
		local startTime = expirationTime - duration

		local thecolor = settings.color
		if (count == 2) then
			thecolor = settings.color2
		elseif (count == 3) then
			thecolor = settings.color3
		end

		if (settings.enable) then
			self.core:SendStatusGained(guid, "alert_GraceStack",
									   settings.priority,
									   (settings.range and 40),
									   thecolor,
									   tostring(count),
									   count,
									   3,
									   icon,
									   startTime,
									   duration,
									   count)
		end

		-- update info for alert_GraceDuration
		expirationTimeList[guid] = expirationTime
		colorList[guid] = thecolor

		-- whenever activeCount is zero, we need to start the repeating event
		if (activeCount == 0 and durationEnabled) then
			self:ScheduleRepeatingEvent("GridGrace_Refresh", f, 0.07)
			self:RefreshActiveUnits()
		end
	else
		self.core:SendStatusLost(guid, "alert_GraceStack")
		if (durationEnabled) then
			self.core:SendStatusLost(guid, "alert_GraceDuration")
		end

		-- update info for alert_GraceDuration
		expirationTimeList[guid] = nil
		colorList[guid] = nil
	end
end


function GridStatusGrace:EnabledStatusCount()
	local enabledStatusCount = 0

	for status, settings in pairs(self.db.profile) do
		if (type(settings) == "table" and settings.enable) then
			enabledStatusCount = enabledStatusCount + 1
		end
	end

	return enabledStatusCount
end

function GridStatusGrace:OnStatusEnable(status)
	self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("UNIT_AURA", "UpdateUnit")

	if (status == "alert_GraceDuration") then
		activeCount = 0
	end

	self:UpdateAllUnits()
end

function GridStatusGrace:OnStatusDisable(status)
	self.core:SendStatusLostAllUnits(status)
	self:UpdateAllUnits()

	if (self:EnabledStatusCount() == 0) then
		self:UnregisterEvent("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_AURA")
	end

	if (status == "alert_GraceDuration") then
		self:CancelScheduledEvent("GridGrace_Refresh")
	end
end
